import pytest
import numpy as np

from disease_detection import cancer  # Replace 'your_module' with the actual module name containing the 'cancer' class


@pytest.fixture
def sample_input():
    return [
        [13.54, 14.36, 87.46, 566.3, 0.09779, 0.08129, 0.06664, 0.04781, 0.1885, 0.05766, 0.2699, 0.7886, 2.058, 23.56,
         0.008462, 0.0146, 0.02387, 0.01315, 0.0198, 0.0023, 15.11, 19.26, 99.7, 711.2, 0.144, 0.1773, 0.239, 0.1288,
         0.2977, 0.07259],
        [19.81, 22.15, 130, 1260, 0.09831, 0.1027, 0.1479, 0.09498, 0.1582, 0.05395, 0.7582, 1.017, 5.865, 112.4,
         0.006494, 0.01893, 0.03391, 0.01521, 0.01356, 0.001997, 27.32, 30.88, 186.8, 2398, 0.1512, 0.315, 0.5372, 0.2388,
         0.2768, 0.07615]
    ]

@pytest.mark.filterwarnings("ignore::UserWarning")
def test_cancer_initialization():
    with pytest.raises(ValueError):
        # Test initialization with None
        g = cancer.cancer(None)

@pytest.mark.filterwarnings("ignore::UserWarning")
def test_cancer_input_validation(sample_input):
    with pytest.raises(ValueError):
        # Test initialization with invalid input length
        g = cancer.cancer(np.array([[1, 2, 3, 4, 5, 6, 7, 8, 9]]))
    
    with pytest.raises(ValueError):
        # Test initialization with non-numeric input
        g = cancer.cancer(np.array([['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h']]))

@pytest.mark.filterwarnings("ignore::UserWarning")
def test_cancer_knn_prediction(sample_input):
    g = cancer.cancer(sample_input)
    result = g.KNearestNeighbours()
    assert isinstance(result, list)

@pytest.mark.filterwarnings("ignore::UserWarning")
def test_cancer_logistic_prediction(sample_input):
    g = cancer.cancer(sample_input)
    result = g.LogisticRegression()
    assert isinstance(result, list)

@pytest.mark.filterwarnings("ignore::UserWarning")
def test_cancer_gnb_prediction(sample_input):
    g = cancer.cancer(sample_input)
    result = g.GNB()
    assert isinstance(result, list)

@pytest.mark.filterwarnings("ignore::UserWarning")
def test_cancer_svc_prediction(sample_input):
    g = cancer.cancer(sample_input)
    result = g.SupportVectorClassifier()
    assert isinstance(result, list)

@pytest.mark.filterwarnings("ignore::UserWarning")
def test_cancer_rf_prediction(sample_input):
    g = cancer.cancer(sample_input)
    result = g.RandomForest()
    assert isinstance(result, list)



import numpy as np
import pytest
from disease_detection import diabetes
@pytest.fixture

def sample_input():
    return np.array([
        [6, 148, 72, 35, 0, 33.6, 0.627, 50],
        [11, 138, 76, 0, 0, 33.2, 0.42, 35],
        [10, 139, 80, 0, 0, 27.1, 1.441, 57]
    ])

def test_diabetes_initialization():
    with pytest.raises(ValueError):
        # Test initialization with None
        g = diabetes.diabetes(None)

def test_diabetes_input_validation(sample_input):
    with pytest.raises(ValueError):
        # Test initialization with invalid input length
        g = diabetes.diabetes(np.array([[1, 2, 3, 4, 5, 6, 7, 8, 9]]))
    
    with pytest.raises(ValueError):
        # Test initialization with non-numeric input
        g = diabetes.diabetes(np.array([['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h']]))

def test_diabetes_knn_prediction(sample_input):
    g = diabetes.diabetes(sample_input)
    result = g.KNearestNeighbours()
    assert isinstance(result, list)

def test_diabetes_svc_prediction(sample_input):
    g = diabetes.diabetes(sample_input)
    result = g.SupportVectorClassifier()
    assert isinstance(result, list)

def test_diabetes_rf_prediction(sample_input):
    g = diabetes.diabetes(sample_input)
    result = g.RandomForest()
    assert isinstance(result, list)


from sklearn.ensemble import RandomForestClassifier
from sklearn.linear_model import LogisticRegression
from sklearn.neighbors import KNeighborsClassifier
from sklearn.metrics import accuracy_score, classification_report
import matplotlib.pyplot as plt
import seaborn as sns
from sklearn.metrics import confusion_matrix

def train_random_forest(X_train, y_train, n_estimators=100, max_depth=None):
    # Train Random Forest classifier
    clf_rf = RandomForestClassifier(n_estimators=n_estimators, max_depth=max_depth, random_state=42)
    clf_rf.fit(X_train, y_train)
    return clf_rf
    
def train_logistic_regression(X_train, y_train, random_state = 0):
    classifier_lr = LogisticRegression(random_state = random_state)
    classifier_lr.fit(X_train, y_train)
    return classifier_lr

def train_k_nearest_neighbours(X_train, y_train, n_neighbors = 5, metric = 'minkowski', p = 2):
    classifier_knn = KNeighborsClassifier(n_neighbors = n_neighbors, metric = metric, p = p)
    classifier_knn.fit(X_train, y_train)
    return classifier_knn

def evaluate_model(clf, X_test, y_test, req_result = 0):
    # Make predictions
    y_pred = clf.predict(X_test)
    # Evaluate the model
    accuracy = accuracy_score(y_test, y_pred)
    report = classification_report(y_test, y_pred)
    if req_result == 0:
        return y_pred, accuracy, report
    elif req_result == 1:
        return y_pred
    elif req_result == 2:
        return accuracy
    elif req_result == 3:
        return report

def plot_confusion_matrix(y_true, y_pred):
    # Plot confusion matrix
    cm = confusion_matrix(y_true, y_pred)
    sns.heatmap(cm, annot=True, fmt='d', cmap='Blues', cbar=False,
                xticklabels=['Benign', 'Malignant'], yticklabels=['Benign', 'Malignant'])
    plt.xlabel('Predicted')
    plt.ylabel('True')
    plt.title('Confusion Matrix')
    plt.show()


import pandas as pd
from sklearn.impute import SimpleImputer
from sklearn.preprocessing import StandardScaler

def preprocess_data(data, is_scale = 0):
    # Impute missing values with mean
    imputer = SimpleImputer(strategy='mean')
    data = pd.DataFrame(imputer.fit_transform(data), columns=data.columns)

    if is_scale == 1:
        scaler = StandardScaler()
        data = pd.DataFrame(scaler.fit_transform(data), columns=data.columns)

    return data



import warnings
warnings.filterwarnings("ignore")

import pandas as pd
import pkg_resources

from sklearn.model_selection import train_test_split
from sklearn.neighbors import KNeighborsClassifier
from sklearn.svm import SVC
from sklearn.ensemble import RandomForestClassifier

resource_package = __name__
resource_path = '/'.join(('datasets', 'diabetes.csv'))  
template = pkg_resources.resource_stream(resource_package, resource_path)

dataset = pd.read_csv(template)

X = dataset.drop(['Outcome'], axis=1)
Y = dataset['Outcome']

X_train, X_test, Y_train, Y_test = train_test_split(X, Y, test_size = 0.25, random_state = 0)

classifier_knn = KNeighborsClassifier(n_neighbors = 8, p = 1)
classifier_knn.fit(X_train, Y_train)

classifier_svc = SVC(kernel = 'poly', random_state = 0)
classifier_svc.fit(X_train, Y_train)

classifier_rf = RandomForestClassifier(n_estimators = 15, criterion = 'entropy', random_state = 0)
classifier_rf.fit(X_train, Y_train)

class diabetes:
    def __init__(self, X_inp):
        """Initialize the class"""
        self.flag = 0
        if X_inp is not None:
            for i in X_inp:
                if len(i) != 8:
                    self.flag = 1
                    raise ValueError("Input array length must be 8 for all rows")
        
                if not all(isinstance(element, (int, float)) for element in i):
                    self.flag = 1
                    raise ValueError("All elements in the input array must be numeric")
        else:
            self.flag = 1
            raise ValueError("Input cannot be None")
        
        if self.flag == 0:
            self.X_inp = X_inp
    
    def KNearestNeighbours(self):
        if self.flag != 0:
            return 'Initialization failed'
        return list(classifier_knn.predict(self.X_inp)) 
    
    def SupportVectorClassifier(self):
        if self.flag != 0:
            return 'Initialization failed'
        return list(classifier_svc.predict(self.X_inp)) 
    
    def RandomForest(self):
        if self.flag != 0:
            return 'Initialization failed'
        return list(classifier_rf.predict(self.X_inp))


from setuptools import setup, find_packages

setup(
    name='disease_detection',
    version='0.2.0',
    packages=find_packages(),
    package_data={'disease_detection': ['datasets/cancer.csv', 'datasets/diabetes.csv']},
    install_requires=[
        'numpy',
        'matplotlib',
        'pandas',
        'scikit-learn',
        'seaborn',
        'pytest'
    ],
    
    author='Niraj Prasad',
    author_email='nirajsaiprasad@gmail.com',
    description='A package for disease detection',
    #long_description=open('README.md').read(),
    #long_description_content_type='text/markdown',
    url='https://github.com/nirajprasad12/ds5010-disease-detection',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
)



from sklearn.feature_selection import SelectKBest, f_classif

def select_best_features(X, y, k=10):
    # Select top k features based on ANOVA F-statistic
    selector = SelectKBest(score_func=f_classif, k=k)
    X_selected = selector.fit_transform(X, y)

    return X_selected
